/********************************************************************************* 
 *     COPYRIGHT NOTICE:
 *     Copyright © 2013 Andrew Smith (GNA SERVICES INC) <Andrew@GNAServicesInc.com>
 *     All Rights Reserved.
 *
 *   This file, user_settings.c , is part of tpad.
 *
 *   tpad is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *   tpad is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with tpad.  If not, see <http://www.gnu.org/licenses/>.
 ********************************************************************************/
#include "user_settings.h"
cfgSet userCfg;
void cfg_on_exit(){
	config_destroy(&userCfg.cfg);
}
int cfg_show_full_path(){
	return(userCfg.show_full_path);
}
int cfg_lang(){
return(userCfg.show_lang);
}
int cfg_line(){
return(userCfg.show_line_numbers);
}
int cfg_spell(){
return(userCfg.show_spelling_errors);
}
int cfg_sWidth(){
return(userCfg.screenWidth);
}
int cfg_sHeight(){
return(userCfg.screenHeight);
}
int cfg_wWidth(){
return(userCfg.defualt_window_width);
}
int cfg_wHeight(){
return(userCfg.default_window_height);
}
int cfg_undo(){
return(userCfg.undo_level);
}
int cfg_line_wrap(){
return(userCfg.line_wrap);
}
int cfg_gaveUp(){
return(userCfg.gave_up);
}
#ifdef AUTO_TAB_TOGGLE
int cfg_auto_tab(){
return(userCfg.auto_tab);
}
#endif

void cfg_set_show_line(int opt){
	if(opt >= 0 && opt <=1) userCfg.show_line_numbers=opt;
	else gerror_warn(_ERROR_SETTING_SETTING,"cfg_set_show_line",0,0);
}
void cfg_set_show_lang(int opt){
	if(opt >= 0 && opt <=1) userCfg.show_lang=opt;
	else gerror_warn(_ERROR_SETTING_SETTING,"cfg_set_show_lang",0,0);
}
void cfg_set_show_full_path(int opt){
	if(opt >= 0 && opt <=1) userCfg.show_full_path=opt;
	else gerror_warn(_ERROR_SETTING_SETTING,"cfg_show_full_path",0,0);
}
void cfg_set_show_spelling(int opt){
	if(opt >= 0 && opt <=1) userCfg.show_spelling_errors=opt;
	else gerror_warn(_ERROR_SETTING_SETTING,"cfg_set_show_spelling",0,0);
}
void cfg_set_screen_width(int opt){
	if(opt > 0) userCfg.screenWidth=opt;
	else gerror_warn(_ERROR_SETTING_SETTING,"cfg_set_show_spelling",0,0);
}
void cfg_set_screen_height(int opt){
	if(opt > 0) userCfg.screenHeight=opt;
	else gerror_warn(_ERROR_SETTING_SETTING,"cfg_set_show_spelling",0,0);
}
void cfg_set_default_width(int opt){
	if(opt >= 25) userCfg.defualt_window_width=opt;
	else gerror_warn(_ERROR_TOO_SMALL,"cfg_set_default_width",0,0);
}
void cfg_set_default_height(int opt){
	if(opt >= 50) userCfg.default_window_height=opt;
	else gerror_warn(_ERROR_TOO_SMALL,"cfg_set_default_height",0,0);
}
void cfg_set_undo(int opt){
	if(opt >= 0) userCfg.undo_level=opt;
	else gerror_warn(_ERROR_SETTING_SETTING,"cfg_set_undo",0,0);
}
void cfg_set_show_line_wrap(int opt){
	if(opt >= 0 && opt <=1) userCfg.line_wrap=opt;
	else gerror_warn(_ERROR_SETTING_SETTING,"cfg_set_show_line_wrap",0,0);
}
#ifdef AUTO_TAB_TOGGLE
void cfg_set_auto_tab(int opt){
	if(opt >= 0 && opt <=1) userCfg.auto_tab=opt;
	else gerror_warn(_ERROR_SETTING_SETTING,"cfg_set_auto_tab",0,0);
}
#endif

void config_setup(char *cfg_file){
	config_init(&userCfg.cfg);
	userCfg.root = config_root_setting(&userCfg.cfg);
	userCfg.show_line_numbers=1;
	userCfg.show_lang=0;
	userCfg.show_spelling_errors=0;
	userCfg.defualt_window_width=_DEFAULT_WINDOW_WIDTH;
	userCfg.default_window_height=_DEFAULT_WINDOW_HEIGHT;
	userCfg.undo_level=0;
	userCfg.line_wrap=1;
	userCfg.show_full_path=1;
	#ifdef AUTO_TAB_TOGGLE
	userCfg.auto_tab=0;
	#endif
	userCfg.gave_up=0;

	if(access(cfg_file, R_OK|W_OK ) != -1) cfg_read_in(cfg_file);
	else userCfg.gave_up=store_settings(cfg_file,
	                                    userCfg.show_line_numbers,
	                                    userCfg.show_lang,	                               
	                                    userCfg.show_spelling_errors,
	                                    userCfg.defualt_window_width,
	                                    userCfg.default_window_height,
	                                    _DEFAULT_WINDOW_WIDTH,
	                                    _DEFAULT_WINDOW_HEIGHT,
	                                    userCfg.undo_level,
	                                    userCfg.line_wrap,
	                                    userCfg.show_full_path
	                                    #ifdef AUTO_TAB_TOGGLE
	                                    ,
	                                    userCfg.auto_tab
										#endif
	                                    );
										
}

void cfg_read_in(char *cfg_file){
	// Range of return values are valid. Values are checked in each function and
	// set to a valid default if they are not in range.  
	userCfg.show_line_numbers=cfg_get_show_line_numbers(cfg_file);	
	userCfg.show_spelling_errors=cfg_get_check_spellcheck(cfg_file);
	userCfg.show_lang=cfg_get_default_show_lang(cfg_file);
	userCfg.defualt_window_width=cfg_get_default_window_width(cfg_file);
	userCfg.default_window_height=cfg_get_default_window_height(cfg_file);	
	userCfg.undo_level=cfg_get_def_undo_level(cfg_file);
	userCfg.line_wrap=cfg_get_show_line_wrap(cfg_file);
	userCfg.show_full_path=cfg_get_show_full_path(cfg_file);
	#ifdef AUTO_TAB_TOGGLE
	userCfg.auto_tab=cfg_get_default_auto_tab(cfg_file);
	#endif
}

int cfg_get_def_undo_level(char *cfg_file){
	if(!config_read_file(&userCfg.cfg, (char*)cfg_file )) return(0);
	else {
		int DefaultUndo;	
		userCfg.setting = config_lookup(&userCfg.cfg,"User_Interface_Settings");
		config_setting_lookup_int(userCfg.setting,"Undo_Level", &DefaultUndo);
		if (DefaultUndo < 0) DefaultUndo=0;
		return DefaultUndo;
	}
}
int cfg_get_default_window_width(char *cfg_file){
	if(!config_read_file(&userCfg.cfg, (char*)cfg_file )) return(_DEFAULT_WINDOW_WIDTH);	
	else {
		int DefaultWindowWidth;	
		userCfg.setting = config_lookup(&userCfg.cfg,"User_Interface_Settings");
		config_setting_lookup_int(userCfg.setting,"Default_Window_Width", &DefaultWindowWidth);
		if(DefaultWindowWidth < 25 || DefaultWindowWidth > userCfg.screenWidth) DefaultWindowWidth=_DEFAULT_WINDOW_WIDTH;
		return DefaultWindowWidth;
	}
}
int cfg_get_default_window_height(char *cfg_file){
	if(!config_read_file(&userCfg.cfg, (char*)cfg_file )) return(_DEFAULT_WINDOW_HEIGHT);
	else {
		int DefaultWindowHeight;	
		userCfg.setting = config_lookup(&userCfg.cfg,"User_Interface_Settings");
		config_setting_lookup_int(userCfg.setting,"Default_Window_Height", &DefaultWindowHeight);
		if(DefaultWindowHeight < 50 ||DefaultWindowHeight > userCfg.screenHeight) DefaultWindowHeight=_DEFAULT_WINDOW_HEIGHT;
	return DefaultWindowHeight;
	}
}
int cfg_get_check_spellcheck(char *cfg_file){
	if(!config_read_file(&userCfg.cfg, (char*)cfg_file )) return (0);	
	else {
		int CheckSpelling;	
		userCfg.setting = config_lookup(&userCfg.cfg,"User_Pref");
		config_setting_lookup_int(userCfg.setting,"Spelling", &CheckSpelling);
		if (CheckSpelling < 0 || CheckSpelling > 1) CheckSpelling=0;
	return CheckSpelling;
	}
}
int cfg_get_show_line_numbers(char *cfg_file){
	if(!config_read_file(&userCfg.cfg, (char*)cfg_file )) return (1);
	else {
		int ShowLineNumbers;	
		userCfg.setting = config_lookup(&userCfg.cfg,"User_Pref");
		config_setting_lookup_int(userCfg.setting,"LineNumbers", &ShowLineNumbers);
		if (ShowLineNumbers < 0 || ShowLineNumbers > 1) ShowLineNumbers=1;
	return ShowLineNumbers;
	}
}
int cfg_get_default_show_lang(char *cfg_file){
	if(!config_read_file(&userCfg.cfg, (char*)cfg_file )) return(0);
	else {
		int DefaultShowLang;	
		userCfg.setting = config_lookup(&userCfg.cfg,"User_Pref");
		config_setting_lookup_int(userCfg.setting,"ShowLang", &DefaultShowLang);
		if (DefaultShowLang < 0 || DefaultShowLang > 1) DefaultShowLang=0;
	return DefaultShowLang;
	}
}
int cfg_get_show_line_wrap(char *cfg_file){
	if(!config_read_file(&userCfg.cfg, (char*)cfg_file )) return(1);
	else {
		int wrapLines;	
		userCfg.setting = config_lookup(&userCfg.cfg,"User_Pref");
		config_setting_lookup_int(userCfg.setting,"WrapLines", &wrapLines);
		if (wrapLines < 0 || wrapLines > 1) wrapLines=0;
		return wrapLines;
	}
}
int cfg_get_show_full_path(char *cfg_file){
	if(!config_read_file(&userCfg.cfg, (char*)cfg_file )) return(1);
	else {
		int ShowFullPath;	
		userCfg.setting = config_lookup(&userCfg.cfg,"User_Pref");
		config_setting_lookup_int(userCfg.setting,"Show_Full_Path", &ShowFullPath);
		if (ShowFullPath < 0 || ShowFullPath > 1) ShowFullPath=1;
		return ShowFullPath;
	}
}
#ifdef AUTO_TAB_TOGGLE
int cfg_get_default_auto_tab(char *cfg_file){
	if(!config_read_file(&userCfg.cfg, (char*)cfg_file )) return(0);
	else {
		int DefaultAutoTab;	
		userCfg.setting = config_lookup(&userCfg.cfg,"User_Pref");
		config_setting_lookup_int(userCfg.setting,"Auto_Tab", &DefaultAutoTab);
		if (DefaultAutoTab < 0 || DefaultAutoTab > 1) DefaultAutoTab=0;
		return DefaultAutoTab;
	}
}
#endif
